package sk.upjs.projektFyzikov;

import java.awt.Font;

import sk.upjs.jpaz2.Pane;
import sk.upjs.jpaz2.Turtle;

public class NabojPanel extends Pane {
	/**
	 * Korytnacka, ktora bude zapisovat hodnoty do panelu
	 */
	private Turtle pisac;

	/**
	 * Plocha na ktorej sa bude nachadzat panel
	 */
	private Plocha plocha;
	/**
	 * Suradnice vypisujucej korytnacky
	 */
	private int x;
	private int y;
	/**
	 * Castica, ktorej naboj sa bude vypisovat
	 */
	private Castica castica;

	/**
	 * Konstruktor pre "velky" panel, ktory zobrazuje naboj nasledujucej
	 * pridanej castice
	 * 
	 * @param fyzikalnaPlocha
	 *            - plocha poskytujuca udaje na vypisanie
	 */
	public NabojPanel(Plocha fyzikalnaPlocha) {
		super(200, 50);
		setTransparentBackground(false);
		plocha = fyzikalnaPlocha;
		pisac = new Turtle();
		add(pisac);
		pisac.setTransparency(1);
		x = 15;
		y = this.getHeight() / 2 + 5;
		pisac.setPosition(x, y);
		pisac.turn(90);
		pisac.setFont(new Font("Lucida Sans", Font.BOLD, 20));
		pisac.print("Nsobok q = " + plocha.getNaboj());
	}

	/**
	 * Konstruktor pre "maly" panel, ktory zobrazuje naboj castice zobrazenej na
	 * ploche, na ktorej je kurzor, alebo nulu ak kurzor nie je na ziadnej
	 * castici
	 * 
	 * @param fyzikalnaPlocha
	 *            - plocha poskytujuca udaje na vypisanie
	 * @param velkostPisma
	 *            - velkost fontu
	 */
	public NabojPanel(Plocha fyzikalnaPlocha, int velkostPisma) {
		super(200, 50);
		setTransparentBackground(false);
		plocha = fyzikalnaPlocha;
		pisac = new Turtle();
		add(pisac);
		pisac.setTransparency(1);
		x = 5;
		y = this.getHeight() / 2 + 5;
		pisac.setPosition(x, y);
		pisac.turn(90);
		pisac.setFont(new Font("Lucida Sans", Font.BOLD, velkostPisma));

		pisac.print("Nboj = " + plocha.getNaboj());
	}

	/**
	 * Obnovuje hodnotu nasobku q napisanu vo "velkom" panely
	 */
	public void refresh() {
		clear();
		pisac.setPosition(x, y);
		pisac.print("Nsobok q = " + plocha.getNaboj());
	}

	/**
	 * Obnovuje hodnotu naboja napisanu v "malom" panely
	 */
	public void refreshPreCasticu() {
		clear();
		pisac.print("Nboj = " + castica.getNaboj() + " C");
	}

	/**
	 * Resetuje hodnotu naboja na nulu
	 */
	public void resetPreCasticu() {
		clear();
		pisac.setPosition(5, y);
		pisac.print("Nboj = " + 0 + " C");
	}

	public Castica getCastica() {
		return castica;
	}

	public void setCastica(Castica castica) {
		this.castica = castica;
	}
}
