package sk.upjs.projektFyzikov;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Scanner;

import sk.upjs.jpaz2.Pane;
import sk.upjs.jpaz2.Turtle;

public class Plocha extends Pane {
	/**
	 * Nasobok "elementarneho" naboja ulozeneho v triede castica
	 */
	private int naboj = 0;
	/**
	 * List castic zobrazenych na ploche
	 */
	private ArrayList<Castica> castice = new ArrayList<Castica>();
	/**
	 * Castica, ktorou hybeme
	 */
	private Castica tahanaCastica = null;

	private int sirkaPlochy = this.getWidth();
	private int vyskaPlochy = this.getHeight();
	/**
	 * Pocet riadkov a stlpcov myslenej mriezky v ploche, pri "hustote" 1/50
	 */
	private int pocetStlpcov = (int) (this.getWidth() / 50);
	private int pocetRiadkov = (int) (this.getHeight() / 50);
	/**
	 * Vyska a sirka jedneho segmentu mriezky
	 */
	private int sirkaMriezky = (int) (this.getWidth() / pocetStlpcov);
	private int vyskaMriezky = (int) (this.getHeight() / pocetRiadkov);
	/**
	 * Velkost vektora E
	 */
	private double krok = 0;
	/**
	 * Tlacidla na ploche
	 */
	private HardcoreButton hardcoreButton = new HardcoreButton();
	private EButton EButton = new EButton(this, hardcoreButton);
	private DivEButton divEButton = new DivEButton(this);
	private ResetButton resetButton = new ResetButton(this);
	private PridajNaboj nabojButtonPlus = new PridajNaboj(this, true);
	private PridajNaboj nabojButtonMinus = new PridajNaboj(this, false);
	private NabojPanel nabojPanel = new NabojPanel(this);
	private NabojPanel nabojPanel2 = new NabojPanel(this, 16);
	private EPanel EPanel = new EPanel(this);
	private ExitButton exitButton = new ExitButton();
	private SavePositionButton ulozButton = new SavePositionButton(this);
	private LoadPositionButton nacitajButton = new LoadPositionButton(this);
	/**
	 * Pole uchovavajuce vektor E v kazdom priesecniku mriezky
	 */
	private double[][][] mriezka = new double[pocetRiadkov][pocetStlpcov][2];
	/**
	 * Pole uchovavajuce hodnotu divergencie vektora E v kazdom bode plochy
	 */
	private double[][] divergencia = new double[vyskaPlochy][sirkaPlochy];
	/**
	 * Fyzikalna konstanta zabezpecujuca spravny rozmer velicin v sustave SI
	 */
	private static final double jednaLomeneStyriPiEpsilonNula = 1 / (Math.PI * 0.000000000008854187);
	/**
	 * Pole korytnaciek, ktore predstavuju vektory E
	 */
	private Turtle[] sipky = new Turtle[(pocetStlpcov - 1) * (pocetRiadkov - 1)];
	/**
	 * Pole velkosti vektora E
	 */
	private double[] kroky = new double[(pocetStlpcov - 1) * (pocetRiadkov - 1)];

	/**
	 * Konstruktor plochy zabezpecujuci pridanie a spravnu polohu tlacidiel
	 */
	public Plocha() {
		super(800, 600);
		add(divEButton);
		add(EButton);
		add(resetButton);
		add(hardcoreButton);
		add(nabojButtonPlus);
		add(nabojButtonMinus);
		add(nabojPanel);
		add(nabojPanel2);
		add(EPanel);
		add(exitButton);
		add(ulozButton);
		add(nacitajButton);
		divEButton.setPosition(getWidth() - 100, 150);
		resetButton.setPosition(getWidth() - 100, 250);
		EButton.setPosition(getWidth() - 100, 0);
		hardcoreButton.setPosition(getWidth() - 100, 200);
		nabojButtonPlus.setPosition(getWidth() - 100, 50);
		nabojButtonMinus.setPosition(getWidth() - 100, 100);
		exitButton.setPosition(getWidth() - 100, getHeight() - 48);
		nabojPanel.setPosition((getWidth() / 2) - nabojPanel.getWidth() / 2, 0);
		EPanel.setPosition(0, 50);
		ulozButton.setPosition(getWidth() - 100, 300);
		nacitajButton.setPosition(getWidth() - 100, 350);
	}

	/**
	 * Metody, ktore davaju odpoved na otazku, ci sa kurzor nachadza nad
	 * niektorym z tlacidiel
	 * 
	 * @param x
	 *            - suradnica
	 * 
	 * 
	 * @param y
	 *            - suradnica
	 * 
	 * 
	 * @return true ak sa kurzor nachadza nad tlacidlom
	 */
	private boolean jeNadDivE(int x, int y) {
		return (getWidth() - 100 <= x) && (x <= getWidth()) && (150 <= y)
				&& (y <= 198);
	}

	private boolean jeNadE(int x, int y) {
		return (getWidth() - 100 <= x) && (x <= getWidth()) && (0 <= y)
				&& (y <= 48);
	}

	private boolean jeNadPlusButton(int x, int y) {
		return (getWidth() - 100 <= x) && (x <= getWidth()) && (50 <= y)
				&& (y <= 98);
	}

	private boolean jeNadMinusButton(int x, int y) {
		return (getWidth() - 100 <= x) && (x <= getWidth()) && (100 <= y)
				&& (y <= 148);
	}

	private boolean jeNadReset(int x, int y) {
		return (getWidth() - 100 <= x) && (x <= getWidth()) && (250 <= y)
				&& (y <= 298);
	}

	private boolean jeNadHardcoreButton(int x, int y) {
		return (getWidth() - 100 <= x) && (x <= getWidth()) && (200 <= y)
				&& (y <= 248);
	}

	private boolean jeNadNabojPanel(int x, int y) {
		return (((getWidth() / 2) - 100) <= x)
				&& (x <= ((getWidth() / 2) + 100)) && (0 <= y) && (y <= 50);
	}

	private boolean jeNadNabojPanel2(int x, int y) {
		return ((0 <= x) && (x <= (200)) && (0 <= y) && (y <= 50));
	}

	private boolean jeNadEPanel(int x, int y) {
		return ((0 <= x) && (x <= (200)) && (50 <= y) && (y <= 100));
	}

	private boolean jeNadExitButton(int x, int y) {
		return ((getWidth() - 100 <= x) && (x <= getWidth())
				&& (getHeight() - 48 <= y) && (y <= getHeight()));
	}

	private boolean jeNadUlozButton(int x, int y) {
		return ((getWidth() - 100 <= x) && (x <= getWidth()) && (300 <= y) && (y <= 348));
	}

	private boolean jeNadNacitajButton(int x, int y) {
		return ((getWidth() - 100 <= x) && (x <= getWidth()) && (350 <= y) && (y <= 398));
	}

	/**
	 * 
	 * @param x
	 *            - suradnica
	 * 
	 * 
	 * @param y
	 *            - suradnica
	 * 
	 * 
	 * @return True ak sa suradnice bodu nenachadzaju na ziadnom z tlacidiel
	 */
	public boolean MimoTlacidla(int x, int y) {
		return (!jeNadE(x, y)) && (!jeNadDivE(x, y)) && (!jeNadReset(x, y))
				&& (!jeNadHardcoreButton(x, y)) && (!jeNadPlusButton(x, y))
				&& (!jeNadMinusButton(x, y)) && (!jeNadNabojPanel(x, y))
				&& (!jeNadNabojPanel2(x, y)) && (!jeNadEPanel(x, y))
				&& (!jeNadExitButton(x, y)) && (!jeNadNacitajButton(x, y))
				&& (!jeNadUlozButton(x, y));
	}

	/**
	 * Metoda testuje, ktore z tlacidiel E a divE su zapnute. Zapnute moze byt
	 * ktorekolvek tlacidlo, obe alebo ziadne z nich.
	 * 
	 * @param div
	 *            tlacidlo na vykreslenie divergencie E
	 * @param E
	 *            tlacidlo na vykreslenie vektora E
	 * @return stav tlacidiel E a divE
	 */
	public String zapnuteTlacidla(DivEButton div, EButton E) {
		div = divEButton;
		E = EButton;
		if (div.isDivEButtonOn() == true && E.isEButtonOn() == true)
			return "Obe su on";
		if (div.isDivEButtonOn() == true && E.isEButtonOn() == false)
			return "divE je on";
		if (div.isDivEButtonOn() == false && E.isEButtonOn() == true)
			return "E je on";

		return "Obe su off";
	}

	/**
	 * Metoda, ktora prida casticu na zadane suradnice Castica s kladnym nabojom
	 * je reprezentovana cervenou bodkou, castica so zapornym nabojom modrou
	 * bodkou a castica s nulovym nabojom zelenou bodkou
	 * 
	 * @param x
	 * @param y
	 * @param naboj
	 *            pocet "elementarnych" nabojov castice
	 * 
	 * 
	 */
	public void bod(int x, int y, int naboj) {
		Castica Gerlach = new Castica(x, y, naboj);
		this.add(Gerlach);
		// zaznamename novu casticu
		castice.add(Gerlach);

	}

	/**
	 * Uklada polohu a naboj kazdej castice do textoveho suboru
	 * 
	 * @param subor
	 *            do ktoreho sa ulozi konfiguracia castic na ploche
	 */
	public void uloz(File subor) {
		PrintWriter pw = null;
		try {
			pw = new PrintWriter(subor);
			for (Castica n : castice) {
				pw.println(n.toString());
			}
		} catch (FileNotFoundException e) {
			System.err.println("Sbor sa nenaiel");
		} finally {
			if (pw != null) {
				pw.close();
			}
		}
	}

	/**
	 * Vytvori novy list castic na zaklade udajov ulozenych v textovom subore
	 * 
	 * @param subor
	 *            s informaciou o polohach a nabojoch castic
	 * @return list castic na zaklade udajov v subore
	 */
	public static ArrayList<Castica> zoSuboru(File subor) {
		Scanner s = null;
		try {
			ArrayList<Castica> castice = new ArrayList<Castica>();
			s = new Scanner(subor);
			while (s.hasNextLine()) {
				castice.add(Castica.zoStringu(s.nextLine()));
			}
			return castice;
		} catch (FileNotFoundException e) {
			System.err.println("Hadan sbor sa nenaiel");
			return null;
		} finally {
			if (s != null) {
				s.close();
			}
		}
	}

	/**
	 * Ulozi castice z predoslej metody do private premennej castice a prida ich
	 * do plochy
	 * 
	 * @param subor
	 *            s casticami
	 */
	public void nacitaj(File subor) {
		this.castice = zoSuboru(subor);
		for (Castica n : castice) {
			add(n);
		}
	}

	/**
	 * Vycistci plochu od nakresov a vypne tlacidla E a divE ak su zapnute,
	 * castice ostavaju zobrazene na ploche
	 */
	public void reset() {
		clear();
		resetE();
		if (zapnuteTlacidla(divEButton, EButton).equals("Obe su on")) {
			EButton.updateView();
			divEButton.updateView();
		}
		if (zapnuteTlacidla(divEButton, EButton).equals("divE je on"))
			divEButton.updateView();
		if (zapnuteTlacidla(divEButton, EButton).equals("E je on"))
			EButton.updateView();

	}

	/**
	 * Vypne vykreslenie vektora E
	 */
	public void resetE() {
		for (int i = 0; i < sipky.length; i++) {
			remove(sipky[i]);
		}
	}

	/**
	 * Metoda realizujuca klikanie do plochy a funkcionalitu tlacidiel
	 */
	protected void onMouseClicked(int x, int y, MouseEvent detail) {
		if ((jeNadPlusButton(x, y)) || (jeNadMinusButton(x, y)))
			nabojPanel.refresh();
		if (detail.getButton() == MouseEvent.BUTTON1) {
			// Ak klikneme pri zatlacenom tlacidle ALT pridame casticu s opacnym
			// nabojom
			if (detail.isAltDown()) {
				naboj = -naboj;
				nabojPanel.refresh();
			}
			if (MimoTlacidla(x, y)) {
				this.bod(x, y, naboj);
				if (divEButton.isDivEButtonOn()) {
					clear();
					resetE();
					divEButton.updateView();
				}
				if (EButton.isEButtonOn()) {
					EButton.updateView();
					clear();
					resetE();
				}
			}
		}
		/**
		 * Kliknutie pravym tlacidlom na casticu ju vymaze
		 */
		if (detail.getButton() == MouseEvent.BUTTON3) {
			// Ak sa kliklo na niektoru z castic, zoradime list castic podla
			// velkosti
			// a odoberieme nulty prvok, t.j. najblizsiu casticu
			if (somBlizkoCastice(x, y, 6)) {
				ArrayList<Castica> blizkeCastice = new ArrayList<Castica>();
				blizkeCastice = castice;
				Collections.sort(blizkeCastice,
						new KomparatorPodlaVzdialenosti(x, y));
				remove(blizkeCastice.get(0));
				blizkeCastice.remove(0);
				castice = blizkeCastice;
				if (EButton.isEButtonOn()) {
					clear();
					resetE();
					if (hardcoreButton.isHardcoreViewOn() == true)
						vykresliE(vypocetHardcoreE(getCastice()));
					else
						vykresliE(vypocetNormalE(castice));
				}
				if (divEButton.isDivEButtonOn()) {
					clear();
					resetE();
					vypocetDivE(castice);
					vykresliDivE(divergencia);
				}
			}
		}
	}

	/**
	 * Kliknutie lavym tlacidlom na casticu ju oznaci na pohyb
	 */
	@Override
	protected void onMousePressed(int x, int y, MouseEvent detail) {
		if (detail.getButton() == MouseEvent.BUTTON1) {
			for (int i = 0; i < this.castice.size(); i++) {
				if (this.castice.get(i).klikloSaNaMna(x, y)) {
					this.tahanaCastica = castice.get(i);
				}
			}
		}
	}

	/**
	 * Pohybujeme casticou po ploche, ak je zapnute tlacidlo E, vektory E sa
	 * dynamicky vykresluju
	 */
	@Override
	protected void onMouseDragged(int x, int y, MouseEvent detail) {
		super.onMouseDragged(x, y, detail);
		if (this.tahanaCastica != null) {
			this.tahanaCastica.setPosition(x, y);
		}
		if (EButton.isEButtonOn()) {
			clear();
			resetE();
			if (hardcoreButton.isHardcoreViewOn() == true)
				vykresliE(vypocetHardcoreE(getCastice()));
			else
				vykresliE(vypocetNormalE(getCastice()));
		}
		if (divEButton.isDivEButtonOn()) {
			clear();
			resetE();
			divEButton.updateView();
		}
	}

	/**
	 * Ak sa kurzor mysi nachadza nad nejakou casticou, na panely s nabojom sa
	 * vypise jej naboj, ak sa nachadza na sipke vektora E, na E - panely sa
	 * vypise |E|
	 */
	@Override
	protected void onMouseMoved(int x, int y, MouseEvent detail) {
		if (somBlizkoCastice(x, y, 6)) {
			ArrayList<Castica> blizkeCastice = new ArrayList<Castica>();
			blizkeCastice = castice;
			Collections.sort(blizkeCastice, new KomparatorPodlaVzdialenosti(x,
					y));
			nabojPanel2.setCastica(blizkeCastice.get(0));
			nabojPanel2.refreshPreCasticu();
		} else {
			nabojPanel2.resetPreCasticu();
		}
		if (EButton.isEButtonOn()) {
			if (somBlizkoSipky(x, y, 5)) {
				krok = this.somBlizkoSipkyAChcemDlzkuVektora(x, y, 5);
				EPanel.refresh();
			} else {
				krok = 0;
				EPanel.clear();
			}
		}
	}

	/**
	 * Ak pustime casticu, prestavame ju tahat
	 */
	protected void onMouseReleased(int x, int y, MouseEvent detail) {
		if (detail.getButton() == MouseEvent.BUTTON1) {
			this.tahanaCastica = null;
		}
	}

	/**
	 * Vypocet vektora intenzity elektrickeho pola (E) pri vypnutom hardcore
	 * view, E sa nepocita v tesnej blizkosti castice
	 * 
	 * @param castice
	 *            -list castic zobrazenych na ploche
	 * 
	 * @return pole obsahujuce vypocitane vektory E, vid. instancna premenna
	 *         mriezka
	 */
	public double[][][] vypocetNormalE(ArrayList<Castica> castice) {
		double E1 = 0;
		double E2 = 0;
		for (int i = 1; i < mriezka.length; i++) {
			for (int j = 1; j < mriezka[i].length; j++) {
				for (int k = 0; k < castice.size(); k++) {
					// x-ova zlozka r
					int x = j * sirkaMriezky;
					// y-ova zlozka r
					int y = i * vyskaMriezky;
					if (somBlizkoCastice(x, y, 30) == true)
						continue;
					// x - xk
					double deltaX = x - castice.get(k).getX();
					// y - yk
					double deltaY = y - castice.get(k).getY();
					// naboj k-tej castice
					double q = castice.get(k).getNaboj();
					// citatel Eij x-ovej zlozky
					double Xcitatel = jednaLomeneStyriPiEpsilonNula * q
							* deltaX;
					// citatel Eij y-ovej zlozky
					double Ycitatel = jednaLomeneStyriPiEpsilonNula * q
							* deltaY;
					// vzdialenost daneho bodu od k-tej castice
					double r = castice.get(k).distanceTo(x, y);
					double menovatel = r * r * r;
					// vypocet Ex
					E1 = E1 + (Xcitatel / menovatel);
					// vypocet Ey
					E2 = E2 + (Ycitatel / menovatel);
				}
				mriezka[i][j][0] = E1;
				mriezka[i][j][1] = E2;
				E1 = 0;
				E2 = 0;
			}
		}
		return mriezka;
	}

	/**
	 * Vypocet vektora intenzity elektrickeho pola (E) pri zapnutom hardcore
	 * view, E sa pocita aj v tesnej blizkosti castice
	 * 
	 * @param castice
	 *            -list castic
	 * 
	 * @return mriezka
	 */
	public double[][][] vypocetHardcoreE(ArrayList<Castica> castice) {
		double E1 = 0;
		double E2 = 0;
		for (int i = 1; i < mriezka.length; i++) {
			for (int j = 1; j < mriezka[i].length; j++) {
				for (int k = 0; k < castice.size(); k++) {
					double x = j * sirkaMriezky;
					double y = i * vyskaMriezky;
					double deltaX = x - castice.get(k).getX();
					double deltaY = y - castice.get(k).getY();
					double q = castice.get(k).getNaboj();
					double Xcitatel = jednaLomeneStyriPiEpsilonNula * q
							* deltaX;
					double Ycitatel = jednaLomeneStyriPiEpsilonNula * q
							* deltaY;
					double r = castice.get(k).distanceTo(x, y);
					double menovatel = r * r * r;
					E1 = E1 + (Xcitatel / menovatel);
					E2 = E2 + (Ycitatel / menovatel);
				}
				mriezka[i][j][0] = E1;
				mriezka[i][j][1] = E2;
				E1 = 0;
				E2 = 0;
			}
		}
		return mriezka;
	}

	/**
	 * Metoda vykresli vektory E na zaklade vypoctu. Vektory su znazornene
	 * sipkami - korytnackami, dlzka ciary zodpoveda velkosti vektora
	 * 
	 * @param mriezka
	 *            - pole uchovavajuce vektory E
	 * 
	 */
	public void vykresliE(double[][][] mriezka) {
		for (int i = 0; i < sipky.length; i++) {
			sipky[i] = new Turtle();
			this.add(sipky[i]);
			sipky[i].penUp();
		}
		int k = 0;
		// V kazdom bode, v ktorom je vypocitany vektor E korytnacka natocena
		// spravnym smerom urobi krok dlzky velkosti daneho vektora
		for (int i = 1; i < pocetRiadkov; i++) {
			for (int j = 1; j < pocetStlpcov; j++) {
				double x = j * sirkaMriezky;
				double y = i * vyskaMriezky;
				double E1 = mriezka[i][j][0];
				double E2 = mriezka[i][j][1];
				if (Math.sqrt(E1 * E1 + E2 * E2) == 0) {
					remove(sipky[k]);
					k++;
					continue;
				}

				this.add(sipky[k]);
				sipky[k].setPosition(x, y);
				sipky[k].penDown();
				sipky[k].turnTowards(x + E1, y + E2);
				sipky[k].moveTo(x + E1, y + E2);
				kroky[k] = Math.sqrt(E1 * E1 + E2 * E2);
				// Sipky sa nezobrazuju na tlacidlach
				if (!MimoTlacidla((int) sipky[k].getX(), (int) sipky[k].getY()))
					remove(sipky[k]);
				k++;
			}
		}
	}

	/**
	 * Metoda, ktora otestuje, ci existuje castica, ktorej vzdialenost od daneho
	 * bodu je mensia alebo rovna ako parameter Pouzitie : test, ci je dany bod
	 * "blizko" nejakej castice
	 * 
	 * @param x
	 *            -x suradnica bodu
	 * 
	 * @param y
	 *            -y suradnica bodu
	 * 
	 * @param vzdialenost
	 * @return true ak existuje castica, ktorej vzdialenost od daneho bodu je
	 *         mensia ako parameter
	 */
	public boolean somBlizkoCastice(int x, int y, double vzdialenost) {
		for (Castica castica : castice) {
			double vzdialenostKCastici = castica.distanceTo(x, y);
			if (vzdialenostKCastici <= vzdialenost)
				return true;
		}
		return false;
	}

	/**
	 * Metoda, ktora otestuje, ci existuje sipka, ktorej vzdialenost od daneho
	 * bodu je mensia alebo rovna ako parameter Pouzitie : test, ci je dany bod
	 * "blizko" hrotu nejakeho vektora
	 * 
	 * @param x
	 *            -x suradnica bodu
	 * 
	 * @param y
	 *            -y suradnica bodu
	 * 
	 * @param vzdialenost
	 * @return true ak existuje sipka, ktorej vzdialenost od daneho bodu je
	 *         mensia ako parameter
	 */
	public boolean somBlizkoSipky(int x, int y, double vzdialenost) {
		for (int i = 0; i < sipky.length; i++) {
			double vzdialenostKSipke = sipky[i].distanceTo(x, y);
			if (vzdialenostKSipke <= vzdialenost)
				return true;
		}
		return false;
	}

	/**
	 * Metoda, ktora zabezpecuje ziskanie velkosti vektora E, ak mame kurzor na
	 * jeho hrote
	 * 
	 * @param x
	 *            - suradnica
	 * @param y
	 *            - suradnica
	 * @param vzdialenost
	 *            bodu k sipke
	 * @return velkost vektora, ktory je dostatocne blizko danemu bodu
	 */
	public double somBlizkoSipkyAChcemDlzkuVektora(int x, int y,
			double vzdialenost) {
		double d = vzdialenost;
		int k = 0;
		for (int i = 0; i < sipky.length; i++) {
			double vzdialenostKSipke = sipky[i].distanceTo(x, y);
			if (vzdialenostKSipke <= d) {
				d = vzdialenostKSipke;
				k = i;
			}
		}
		return kroky[k];
	}

	/**
	 * Kazdemu bodu plochy priradi divergenciu elektrickeho pola v danom bode
	 * 
	 * @param castice
	 *            - list castic
	 * 
	 *            Metoda meni instancnu premmennu divergencia
	 * 
	 */
	public void vypocetDivE(ArrayList<Castica> castice) {

		double divE = 0;
		// y suradnica
		for (int i = 0; i < vyskaPlochy; i++) {
			// x suradnica
			for (int j = 0; j < sirkaPlochy; j++) {
				for (int k = 0; k < castice.size(); k++) {
					// "na castici" nepocitame divergenciu, pretoze vieme, ze je
					// +- nekonecno
					if (somBlizkoCastice(j, i, 5) == true)
						continue;
					double deltaX = (j - castice.get(k).getX());
					double deltaY = (i - castice.get(k).getY());
					double q = castice.get(k).getNaboj();
					double r = castice.get(k).distanceTo(j, i);
					double menovatel1 = r * r * r;
					double menovatel2 = r * r * r;
					double citatelX = 3 * (deltaX * deltaX);
					double citatelY = 3 * (deltaY * deltaY);

					divE += (jednaLomeneStyriPiEpsilonNula * q * ((2 / menovatel1)
							- (citatelX / menovatel2) - (citatelY / menovatel2)));
				}
				divergencia[i][j] = -divE;
				divE = 0;
			}
		}

	}

	/**
	 * Metoda, ktora sluzi na najdenie maximalneho alebo minimalneho prvku
	 * dvojrozmerneho pola
	 * 
	 * @param pole
	 *            - lubovolne dvojrozmerne pole
	 * 
	 * @param maximum
	 *            - ak je true, hladame maximum, false - hladame minimum
	 * 
	 * @return maximum/minimum pola
	 */
	public double najPrvok(double[][] pole, boolean maximum) {
		double vysledok = 0;
		for (int i = 0; i < pole.length; i++) {
			for (int j = 0; j < pole[i].length; j++) {
				if (maximum == true) {
					if (vysledok < pole[i][j])
						vysledok = pole[i][j];
				} else {
					if (vysledok > pole[i][j])
						vysledok = pole[i][j];
				}
			}
		}
		return vysledok;
	}

	/**
	 * Metoda, ktora vizualizuje divergenciu E vypocitanu predoslou metodou
	 * 
	 * @param divergencia
	 *            - pole obsahujuce hodnotu divE v kazdom bode plochy Cervena
	 *            farba - kladne znamienko divergencie (vektore E v tomto bode
	 *            vteka do objemu) Modra farba zaporne znamienko divergencie (
	 *            vektore E v tomto bode vyteka z objemu) sytejsi farebny odtien
	 *            znamena vyssiu absolutnu hodnotu divergencie
	 */
	public void vykresliDivE(double[][] divergencia) {
		double maximum = this.najPrvok(divergencia, true);
		double minimum = this.najPrvok(divergencia, false);
		double absolutneMaximum = Math
				.max(Math.abs(maximum), Math.abs(minimum));
		// Korytnacka, ktora zafarbi kazdy pixel plochy pozadovanym odtienom
		// modrej alebo cervenej
		Turtle maxwell = new Turtle();
		add(maxwell);
		for (int i = 0; i < divergencia.length; i++) {
			for (int j = 0; j < divergencia[i].length; j++) {
				// Konkretny odtien zavisi od pomeru hodnoty divergencie v danom
				// bode a absolutneho maxima
				double stupenBielej = Math.abs(divergencia[i][j]
						/ absolutneMaximum);
				if (stupenBielej != 0) {
					if (divergencia[i][j] > 0) {
						// Castice nie je potrebne zafarbovat
						if (!somBlizkoCastice(j, i, 5))
							// Farba je cervena a jej transparentnost zavisi od
							// stupna bielej
							maxwell.setFillColor(new Color(1f, 0, 0,
									(float) stupenBielej));
					} else {
						if (!somBlizkoCastice(j, i, 5))
							// Farba je modra a jej transparentnost zavisi od
							// stupna bielej
							maxwell.setFillColor(new Color(0, 0, 1f,
									(float) stupenBielej));
					}
					maxwell.setPosition(j, i);
					maxwell.dot(1);
				}
			}
		}
		remove(maxwell);
	}

	/*
	 * /** Metoda v danom bode vypise hodnotu divergencie v nom, bohuzial pri
	 * tejto konfiguracii nie je tato metoda pouzitelna
	 * 
	 * @param x - suradnica bodu
	 * 
	 * @param y - suradnica bodu
	 * 
	 * @param divergencia - pole uchovavajuce hodnoty divergencie
	 * 
	 * public void vypisDivE(int x, int y, double[][] divergencia){ Turtle pisar
	 * = new Turtle(); this.add(pisar); pisar.turn(90); pisar.setPosition(x, y);
	 * pisar.print("divE = " + divergencia[y][x]); remove(pisar); }
	 */

	/**
	 * Vyprazdni plochu aj list castic, vypne talcidla na vykreslenie E a divE
	 * ak su zapnute, resetuje naboj na nulu
	 */
	public void resetAll() {
		for (int i = 0; i < castice.size(); i++) {
			remove(castice.get(i));
		}
		if (divEButton.isDivEButtonOn())
			divEButton.updateView();
		if (EButton.isEButtonOn())
			EButton.updateView();
		clear();
		castice.clear();
		naboj = 0;
		nabojPanel.refresh();
		EPanel.clear();
		resetE();
	}

	// Nasleduju potrebne gettre a settre
	public ArrayList<Castica> getCastice() {
		return castice;
	}

	public void setCastice(ArrayList<Castica> castice) {
		this.castice = castice;
	}

	public double[][][] getMriezka() {
		return mriezka;
	}

	public void setMriezka(double[][][] mriezka) {
		this.mriezka = mriezka;
	}

	public int getSirkaMriezky() {
		return sirkaMriezky;
	}

	public void setSirkaMriezky(int sirkaMriezky) {
		this.sirkaMriezky = sirkaMriezky;
	}

	public int getVyskaMriezky() {
		return vyskaMriezky;
	}

	public void setVyskaMriezky(int vyskaMriezky) {
		this.vyskaMriezky = vyskaMriezky;
	}

	public int getNaboj() {
		return naboj;
	}

	public void setNaboj(int naboj) {
		this.naboj = naboj;
	}

	public double[][] getDivergencia() {
		return divergencia;
	}

	public double getKrok() {
		return krok;
	}

}
